import peasy.*;
import controlP5.*;
import processing.opengl.*;

PeasyCam cam;
ControlP5 cp5set1;
ControlP5 cp5set2;


//sliders
Slider inclineSlider;
Slider scaleSlider;
Slider helixSpaceSlider;
Slider opacitySlider;

//data
Table table;
int rowNums;
ArrayList<CD> cds = new ArrayList<CD>();


//menu items
String[] genres = {"Jazz","Rock","Pop","Country","Folk","Electronic","Soul","Blues","Rap","Musical"};

boolean jazzSelect = true;
boolean rockSelect = true;
boolean popSelect = true;
boolean countrySelect = true;
boolean folkSelect = true;
boolean electronicSelect = true;
boolean soulSelect = true;
boolean bluesSelect = true;
boolean rapSelect = true;
boolean musicalSelect = true;

boolean displayDates = false;
boolean displayMenu = true;


//colors
color jazzColor = color(255,255,0);
color rockColor = color(0,255,255);
color popColor = color(255,0,255);
color countryColor = color(255,255,255);
color folkColor = #F2DF55;
color electronicColor = #1FD7C2;
color soulColor = #8AE481;
color bluesColor =#62BDE5;
color rapColor = #BB97D0;
color musicalColor = #DF7591;


//numbers
float radius = 80;
float rectBase = .9*radius*sin(2*PI/365);
float incline = .3;
float scale = 100;
float helixSpace = 4;
float opacity = 160;

int menuWidth = 300;
int buttonWidth = 80;
int buttonHeight = 20;
int buttonX = 100;

int genreNums = genres.length;
int buttonSpace = 30;









//-----------------------------------------------
void setup() {
  size(1920, 980,OPENGL);
  cam = new PeasyCam(this, 200);
  
  //collect data from csv
  table = loadTable("sheppard3dprojectCD.csv","header");
  println(table.getRow(0));
  rowNums = table.getRowCount();
  for (int i = 0; i<rowNums; i++) {
    String date = table.getString(i, 0); 
    float jazz = log(table.getInt(i, 1)+1);  //data collected on log scale
    float rock = log(table.getInt(i, 2)+1); 
    float pop = log(table.getInt(i, 3)+1);
    float country = log(table.getInt(i, 4)+1);
    float folk = log(table.getInt(i, 5)+1);
    float electronic = log(table.getInt(i, 6)+1);
    float soul = log(table.getInt(i, 7)+1);
    float blues = log(table.getInt(i, 8)+1);
    float rap = log(table.getInt(i, 9)+1);
    float musical = log(table.getInt(i, 10)+1);
    cds.add(new CD(date, jazz, rock, pop,country,folk,electronic,soul,blues,rap,musical));
  }
  
  
  //expanded menu controls
  cp5set1 = new ControlP5(this);
  for (int i = 0; i<genreNums; i++) {
    cp5set1.addButton(genres[i], 0,    buttonX, 60 + i*buttonSpace, buttonWidth, buttonHeight).setId(i);
  } 
  cp5set1.addButton("Dates", 4, buttonX, 390,buttonWidth, buttonHeight).setId(12);
  cp5set1.addButton("<", 10, menuWidth, 500, 20, 80).setId(10);
  cp5set1.setAutoDraw(false);
  
  //collapsed menu controls
  cp5set2 = new ControlP5(this);
  cp5set2.addButton(">", 10, 0, 500, 20, 80).setId(10);
  cp5set2.setAutoDraw(false);

  
  //sliders
  inclineSlider = cp5set1.addSlider("incline");
  inclineSlider.setRange(0,2)
                .setSize(100,30)
                .setPosition(80,600)
                .setColorActive(color(255,0,0))
                .setColorBackground(color(100,0,0))
                .setColorForeground(color(200,0,0));
                
  scaleSlider = cp5set1.addSlider("scale");
  scaleSlider.setRange(1,300)
                .setSize(100,30)
                .setPosition(80,700)
                .setColorActive(color(255,0,0))
                .setColorBackground(color(100,0,0))
                .setColorForeground(color(200,0,0));
                
  helixSpaceSlider = cp5set1.addSlider("helixSpace");
  helixSpaceSlider.setRange(1,10)
                .setSize(100,30)
                .setPosition(80,800)
                .setColorActive(color(255,0,0))
                .setColorBackground(color(100,0,0))
                .setColorForeground(color(200,0,0));
                
  opacitySlider = cp5set1.addSlider("opacity");
  opacitySlider.setRange(0,255)
                .setSize(100,30)
                .setPosition(80,900)
                .setColorActive(color(255,0,0))
                .setColorBackground(color(100,0,0))
                .setColorForeground(color(200,0,0));                               

}


void draw(){
  background(50);
  smooth();

  noFill();
  stroke(180,180,180,180);
  strokeWeight(2);
  
  //helix outline
  beginShape();
  for (int i = -1; i < rowNums; i++){
    curveVertex((radius+scale)*cos(i*2*PI/365), (radius+scale)*sin(i*2*PI/365),(-rowNums/2 +i)*incline);
  }
  //endShape();
  //for (int i = -1; i < rowNums; i++){
  //  curveVertex((radius+0.67*scale)*cos(i*2*PI/365), (radius+0.67*scale)*sin(i*2*PI/365),(-rowNums/2 +i)*incline);
  //}
  
  //for (int i = -1; i < rowNums; i++){
  //  curveVertex((radius+0.33*scale)*cos(i*2*PI/365), (radius+0.33*scale)*sin(i*2*PI/365),(-rowNums/2 +i)*incline);
  //}
  endShape();

  strokeWeight(0);
  
  
  
  //display genre data and dates
  for (int i = 0; i < rowNums - 1; i++) {

    pushMatrix();
    translate(0,0,(-rowNums/2 +i)*incline);
    rotateZ(i*2*PI/365);
    rotateX(0.1);
    translate(radius,-rectBase/2,0);
    
    float jazzLength = map(cds.get(i).jazz, 0, log(1000), 0, scale+1);
    float rockLength = map(cds.get(i).rock, 0, log(1000), 0, scale);
    float popLength = map(cds.get(i).pop, 0, log(1000), 0, scale);
    float countryLength = map(cds.get(i).country, 0, log(1000), 0, scale);
    float folkLength = map(cds.get(i).folk, 0, log(1000), 0, scale);
    float electronicLength = map(cds.get(i).electronic, 0, log(1000), 0, scale);
    float soulLength = map(cds.get(i).soul, 0, log(1000), 0, scale);
    float bluesLength = map(cds.get(i).blues, 0, log(1000), 0, scale);
    float rapLength = map(cds.get(i).rap, 0, log(1000), 0, scale);
    float musicalLength = map(cds.get(i).musical, 0, log(1000), 0, scale);
    
    //displaying dates
    if( displayDates == true && dateParser(cds.get(i).date)[2] > dateParser(cds.get(i+1).date)[2]){
      fill(230,230,230,200);
      //rect(2*radius, 0,50,5*rectBase);
          textSize(15);
          text(cds.get(i+1).date,scale + 55,3*rectBase+4);
    }
    
    
    if (jazzSelect == true){
      fill(jazzColor,opacity);
      rect(0, 0,jazzLength,rectBase);
    }
    
    
    if(rockSelect == true){
      pushMatrix();
      translate(0,0,helixSpace);
      fill(rockColor,opacity);
      rect(0, 0,rockLength,rectBase);
      popMatrix();
    }

    if(popSelect == true){
      pushMatrix();
      translate(0,0,2*helixSpace);
      fill(popColor,opacity);
      rect(0, 0,popLength,rectBase);
      popMatrix();
    }
    
    if(countrySelect == true){
      pushMatrix();
      translate(0,0,3*helixSpace);
      fill(countryColor,opacity);
      rect(0, 0,countryLength,rectBase);
      popMatrix();
    }
    
    if(folkSelect == true){
      pushMatrix();
      translate(0,0,4*helixSpace);
      fill(folkColor,opacity);
      rect(0, 0,folkLength,rectBase);
      popMatrix();
    }
    
    if(electronicSelect == true){
      pushMatrix();
      translate(0,0,5*helixSpace);
      fill(electronicColor,opacity);
      rect(0, 0,electronicLength,rectBase);
      popMatrix();
    }
    
    if(soulSelect == true){
      pushMatrix();
      translate(0,0,6*helixSpace);
      fill(soulColor,opacity);
      rect(0, 0,soulLength,rectBase);
      popMatrix();
    }
    
    if(bluesSelect == true){
      pushMatrix();
      translate(0,0,7*helixSpace);
      fill(bluesColor,opacity);
      rect(0, 0,bluesLength,rectBase);
      popMatrix();
    }
    
    if(rapSelect == true){
      pushMatrix();
      translate(0,0,8*helixSpace);
      fill(rapColor,opacity);
      rect(0, 0,rapLength,rectBase);
      popMatrix();
    }
    
    if(musicalSelect == true){
      pushMatrix();
      translate(0,0,9*helixSpace);
      fill(musicalColor,opacity);
      rect(0, 0,musicalLength,rectBase);
      popMatrix();
    }
    
    
    //attempt at rollover interaction
    
    //float mouseObjectDistanceX = abs(mouseX-screenX(jazzLength/2, -rectBase/2, 0));
    //float mouseObjectDistanceY = abs(mouseY-screenY(jazzLength/2, -rectBase/2, 0)); 
    //if (mouseObjectDistanceX < jazzLength/2 && mouseObjectDistanceY < rectBase/2)
    //{
    //  stroke(255,255);
    //  point(radius,-rectBase/2,0);
    //  stroke(0,0,0,0);
    //  fill(255,255,255);
    //  textMode(SHAPE);
    //  textSize(3);
    //  text(cds.get(i).date,0,0);
    //}
    
    popMatrix();

  }
  
  gui();

}



//button controls
void controlEvent(ControlEvent theEvent) {
  int id = theEvent.getController().getId();
  println(id);
      if (id==0) { 
        // the index of checkbox items start at 0
        jazzSelect = !jazzSelect;
      }
      else if(id==1) { 
        // the index of checkbox items start at 0
        rockSelect = !rockSelect;
      }
      else if(id==2) { 
        // the index of checkbox items start at 0
        popSelect = !popSelect;
      }
      else if(id==3) { 
        // the index of checkbox items start at 0
        countrySelect = !countrySelect;
      }
      else if(id==4) { 
        // the index of checkbox items start at 0
        folkSelect = !folkSelect;
      }
      else if(id==5) { 
        // the index of checkbox items start at 0
        electronicSelect = !electronicSelect;
      }
      else if(id==6) { 
        // the index of checkbox items start at 0
        soulSelect = !soulSelect;
      }
      else if(id==7) { 
        // the index of checkbox items start at 0
        bluesSelect = !bluesSelect;
      }
      else if(id==8) { 
        // the index of checkbox items start at 0
        rapSelect = !rapSelect;
      }
      else if(id==9) { 
        // the index of checkbox items start at 0
        musicalSelect = !musicalSelect;
      }
      else if(id==10 || id == 11) { 
        displayMenu = !displayMenu;
      }
      else if(id==12) { 
        displayDates = !displayDates;
      }
}



//menu for toggling visualization
void gui() {
  hint(DISABLE_DEPTH_TEST);
  cam.beginHUD();

  if(displayMenu == true){
    //menu background
    fill(150,150,150,210);
    rect(0,0,menuWidth,1080); 
   
    //genre menu buttons
    if (jazzSelect == true){
      fill(jazzColor);
      rect(buttonX - buttonHeight, 60,buttonHeight,buttonHeight);
    }
    if (rockSelect == true){
      fill(rockColor);
      rect(buttonX - buttonHeight, 90,buttonHeight,buttonHeight);
    }
    if (popSelect == true){
      fill(popColor);
      rect(buttonX - buttonHeight, 120,buttonHeight,buttonHeight);
    }
    if (countrySelect == true){
      fill(countryColor);
      rect(buttonX - buttonHeight, 150,buttonHeight,buttonHeight);
    }
    if (folkSelect == true){
      fill(folkColor);
      rect(buttonX - buttonHeight, 180,buttonHeight,buttonHeight);
    }
    if (electronicSelect == true){
      fill(electronicColor);
      rect(buttonX - buttonHeight, 210,buttonHeight,buttonHeight);
    }
    if (soulSelect == true){
      fill(soulColor);
      rect(buttonX - buttonHeight, 240,buttonHeight,buttonHeight);
    }
    if (bluesSelect == true){
      fill(bluesColor);
      rect(buttonX - buttonHeight, 270,buttonHeight,buttonHeight);
    }
    if (rapSelect == true){
      fill(rapColor);
      rect(buttonX - buttonHeight, 300,buttonHeight,buttonHeight);
    }
    if (musicalSelect == true){
      fill(musicalColor);
      rect(buttonX - buttonHeight, 330,buttonHeight,buttonHeight);
    }
    if (displayDates == true){
      fill(255,255,255,210);
      rect(buttonX - buttonHeight, 390,buttonHeight,buttonHeight);
    }
    
    
    cp5set1.draw();
  } else {
    cp5set2.draw();
  }
  
//void keyPressed() {
//      if (keyCode==(48)) { 
//        // the index of checkbox items start at 0
//        jazzSelect = !jazzSelect;
//      }
//      else if(keyCode==(49)) { 
//        // the index of checkbox items start at 0
//        rockSelect = !rockSelect;
//      }
//      else if(keyCode==(50)) { 
//        // the index of checkbox items start at 0
//        popSelect = !popSelect;
//      }
//      else if(keyCode==(51)) { 
//        // the index of checkbox items start at 0
//        countrySelect = !countrySelect;
//      }
//      else if(keyCode==(52)) { 
//        // the index of checkbox items start at 0
//        folkSelect = !folkSelect;
//      }
//      else if(keyCode==(53)) { 
//        // the index of checkbox items start at 0
//        electronicSelect = !electronicSelect;
//      }
//      else if(keyCode==(54)) { 
//        // the index of checkbox items start at 0
//        soulSelect = !soulSelect;
//      }
//      else if(keyCode==(55)) { 
//        // the index of checkbox items start at 0
//        bluesSelect = !bluesSelect;
//      }
//      else if(keyCode==(56)) { 
//        // the index of checkbox items start at 0
//        rapSelect = !rapSelect;
//      }
//      else if(keyCode==(57)) { 
//        // the index of checkbox items start at 0
//        musicalSelect = !musicalSelect;
//      }
      
//}
                


  //---------------------------
  cam.endHUD();
  hint(ENABLE_DEPTH_TEST);
}
